package jonasl.ime;

import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.content.res.Configuration;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.ListPreference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.preference.PreferenceScreen;

public class PrefsGestures extends PreferenceActivity implements OnSharedPreferenceChangeListener {

	public static String[] entries = {"None", "Delete last word", "Delete last sentence", "Cursor after word", "Hide keyboard", "Shift", "Voice input", "Cycle language", "Cycle keyboard", "Toggle QWERTY prediction"};
	public static String[] values = {"none", "delete", "deleteS", "moveAfter", "hide", "shift", "voice", "cycleLang", "cycleKB", "qwertyPrediction"};
	
	public static String[] entriesNoShift = {"None", "Delete last word", "Delete last sentence", "Hide keyboard", "Voice input", "Cycle language", "Cycle keyboard", "Toggle QWERTY prediction"};
	public static String[] valuesNoShift = {"none", "delete", "deleteS", "hide", "voice", "cycleLang", "cycleKB", "qwertyPrediction"};
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		SharedPreferences prefs = getPreferenceManager().getSharedPreferences();
		PreferenceScreen s = getPreferenceManager().createPreferenceScreen(this);		
		setPreferenceScreen(s);
		
		PreferenceCategory c = new PreferenceCategory(this);
		c.setTitle("Gestures");
		s.addPreference(c);	
		
		CheckBoxPreference swipeWhenComposingPref = new CheckBoxPreference(this);
		swipeWhenComposingPref.setTitle("Swipe when composing");
		swipeWhenComposingPref.setSummary("Enable to allow swipes when dictionary is showing");
		swipeWhenComposingPref.setDefaultValue(true);
		swipeWhenComposingPref.setPersistent(true);
		swipeWhenComposingPref.setKey("swipeWhenComposing");
		
		c.addPreference(swipeWhenComposingPref);
		
		ListPreference lp = new ListPreference(this);
		lp.setKey("swipeUp");
		lp.setTitle("Swipe up");
		lp.setSummary(getEntry(prefs.getString("swipeUp", "none")));
		lp.setDefaultValue("none");
		lp.setEntries(entries);
		lp.setEntryValues(values);
		
		c.addPreference(lp);
		
		lp = new ListPreference(this);
		lp.setKey("swipeDown");
		lp.setTitle("Swipe down");
		lp.setSummary(getEntry(prefs.getString("swipeDown", "none")));
		lp.setDefaultValue("none");
		lp.setEntries(entries);
		lp.setEntryValues(values);
		
		c.addPreference(lp);
		
		lp = new ListPreference(this);
		lp.setKey("swipeLeft");
		lp.setTitle("Swipe left");
		lp.setSummary(getEntry(prefs.getString("swipeLeft", "none")));
		lp.setDefaultValue("none");
		lp.setEntries(entries);
		lp.setEntryValues(values);
		
		c.addPreference(lp);
		
		lp = new ListPreference(this);
		lp.setKey("swipeRight");
		lp.setTitle("Swipe right");
		lp.setSummary(getEntry(prefs.getString("swipeRight", "none")));
		lp.setDefaultValue("none");
		lp.setEntries(entries);
		lp.setEntryValues(values);
		
		c.addPreference(lp);		
		
		lp = new ListPreference(this);
		lp.setKey("tripleComma");
		lp.setTitle("Triple comma");
		lp.setSummary(getEntry(prefs.getString("tripleComma", "none")));
		lp.setDefaultValue("none");
		lp.setEntries(entriesNoShift);
		lp.setEntryValues(valuesNoShift);
		
		c.addPreference(lp);
		
		c = new PreferenceCategory(this);
		c.setTitle("Cycle keyboard");
		s.addPreference(c);
		
		CheckBoxPreference cp = new CheckBoxPreference(this);
		cp.setTitle("QWERTY");
		cp.setKey("kbCycleQ");
		cp.setDefaultValue(true);
		cp.setPersistent(true);
		c.addPreference(cp);
		
		cp = new CheckBoxPreference(this);
		cp.setTitle("Compact QWERTY");
		cp.setKey("kbCycleCQ");
		cp.setDefaultValue(true);
		cp.setPersistent(true);
		c.addPreference(cp);
		
		cp = new CheckBoxPreference(this);
		cp.setTitle("Phone keypad");
		cp.setKey("kbCycleP");
		cp.setDefaultValue(true);
		cp.setPersistent(true);
		c.addPreference(cp);
	}
	
	@Override
    public void onConfigurationChanged(Configuration newConfig) {    	
      super.onConfigurationChanged(newConfig);
    }
	
	@Override
	protected void onResume() {
		super.onResume();
		getPreferenceScreen().getSharedPreferences().registerOnSharedPreferenceChangeListener(this);
	}

	@Override
	protected void onPause() {
		super.onPause();
		getPreferenceScreen().getSharedPreferences().unregisterOnSharedPreferenceChangeListener(this);
	}	
	
	@Override
	public void onSharedPreferenceChanged(SharedPreferences prefs, String key) {
		if(key.equals("swipeWhenComposing") || key.startsWith("kbCycle"))
			return;
		
		getPreferenceScreen().findPreference(key).setSummary(getEntry(prefs.getString(key, "error")));
	}
	
	private String getEntry(String value) {
		for(int i = 0; i < values.length; i++) {
			if(values[i].equals(value))
				return entries[i];
		}
		
		return "Internal error";
	}
	
	public static int getActionCode(String value) {
		for(int i = 0; i < values.length; i++) {
			if(values[i].equals(value))
				return i;
		}
		
		return -1;
	}
}
