package jonasl.ime;

import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.content.res.Configuration;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.text.InputType;
import android.view.inputmethod.EditorInfo;

public class PrefsSoundVibration extends PreferenceActivity implements OnSharedPreferenceChangeListener, OnPreferenceChangeListener {

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		PreferenceScreen s = getPreferenceManager().createPreferenceScreen(this);		
		setPreferenceScreen(s);
		
		PreferenceCategory c = new PreferenceCategory(this);
		c.setTitle("Sound & vibration");
		s.addPreference(c);
		
		CheckBoxPreference soundLevelTogglePref = new CheckBoxPreference(this);
		soundLevelTogglePref.setTitle("Custom sound level");
		soundLevelTogglePref.setDefaultValue(false);
		soundLevelTogglePref.setPersistent(true);
		soundLevelTogglePref.setKey("soundLevelToggle");

		c.addPreference(soundLevelTogglePref);

		EditTextPreference soundLevelPref = new EditTextPreference(this);
		soundLevelPref.setTitle("Sound volume");
		soundLevelPref.setSummary(s.getSharedPreferences().getString("soundLevel", "40") + "%");
		soundLevelPref.setDialogTitle("Sound volume");
		soundLevelPref.setDialogMessage("Enter an integer between 0 and 100");
		soundLevelPref.setDefaultValue("40");
		soundLevelPref.setPersistent(true);
		soundLevelPref.setKey("soundLevel");
		soundLevelPref.getEditText().setInputType(InputType.TYPE_CLASS_NUMBER);
		soundLevelPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);

		c.addPreference(soundLevelPref);

		CheckBoxPreference ignoreSilentModePref = new CheckBoxPreference(this);
		ignoreSilentModePref.setTitle("Ignore silent mode");
		ignoreSilentModePref.setSummary("Achieved by identifying sound as music. Kill KB to apply changes.");
		ignoreSilentModePref.setDefaultValue(false);
		ignoreSilentModePref.setPersistent(true);
		ignoreSilentModePref.setKey("ignoreSilentMode");

		c.addPreference(ignoreSilentModePref);
		
		CheckBoxPreference customSoundsPref = new CheckBoxPreference(this);
		customSoundsPref.setTitle("Custom sounds");
		customSoundsPref.setSummary("Kill KB to apply any changes.");
		customSoundsPref.setDefaultValue(false);
		customSoundsPref.setPersistent(true);
		customSoundsPref.setKey("customSounds");

		c.addPreference(customSoundsPref);
		
		
		EditTextPreference pathPref = new EditTextPreference(this);
		pathPref.setTitle("Normal key");
		pathPref.setSummary(getPathSummary(s.getSharedPreferences().getString("keySoundNormal", "")));
		pathPref.setDialogTitle("Normal key");
		pathPref.setDialogMessage("Enter path (ex: '/sdcard/sounds/sound.ogg' or '/data/local/sound.ogg'). Leave empty for default.");
		pathPref.setDefaultValue("");
		pathPref.setPersistent(true);
		pathPref.setKey("keySoundNormal");
		pathPref.getEditText().setInputType(InputType.TYPE_CLASS_TEXT);
		pathPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);

		c.addPreference(pathPref);		
		pathPref.setDependency("customSounds");
		
		pathPref = new EditTextPreference(this);
		pathPref.setTitle("Space");
		pathPref.setSummary(getPathSummary(s.getSharedPreferences().getString("keySoundSpace", "")));
		pathPref.setDialogTitle("Space");
		pathPref.setDialogMessage("Enter path (ex: '/sdcard/sounds/sound.ogg' or '/data/local/sound.ogg'). Leave empty for default.");
		pathPref.setDefaultValue("");
		pathPref.setPersistent(true);
		pathPref.setKey("keySoundSpace");
		pathPref.getEditText().setInputType(InputType.TYPE_CLASS_TEXT);
		pathPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);

		c.addPreference(pathPref);		
		pathPref.setDependency("customSounds");
		
		
		pathPref = new EditTextPreference(this);
		pathPref.setTitle("Backspace");
		pathPref.setSummary(getPathSummary(s.getSharedPreferences().getString("keySoundBackspace", "")));
		pathPref.setDialogTitle("Backspace");
		pathPref.setDialogMessage("Enter path (ex: '/sdcard/sounds/sound.ogg' or '/data/local/sound.ogg'). Leave empty for default.");
		pathPref.setDefaultValue("");
		pathPref.setPersistent(true);
		pathPref.setKey("keySoundBackspace");
		pathPref.getEditText().setInputType(InputType.TYPE_CLASS_TEXT);
		pathPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);

		c.addPreference(pathPref);		
		pathPref.setDependency("customSounds");
		
		
		pathPref = new EditTextPreference(this);
		pathPref.setTitle("Enter");
		pathPref.setSummary(getPathSummary(s.getSharedPreferences().getString("keySoundEnter", "")));
		pathPref.setDialogTitle("Enter");
		pathPref.setDialogMessage("Enter path (ex: '/sdcard/sounds/sound.ogg' or '/data/local/sound.ogg'). Leave empty for default.");
		pathPref.setDefaultValue("");
		pathPref.setPersistent(true);
		pathPref.setKey("keySoundEnter");
		pathPref.getEditText().setInputType(InputType.TYPE_CLASS_TEXT);
		pathPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);

		c.addPreference(pathPref);		
		pathPref.setDependency("customSounds");
		

		CheckBoxPreference vibrationLengthTogglePref = new CheckBoxPreference(this);
		vibrationLengthTogglePref.setTitle("Custom vibration length");
		vibrationLengthTogglePref.setDefaultValue(false);
		vibrationLengthTogglePref.setPersistent(true);
		vibrationLengthTogglePref.setKey("vibrationLengthToggle");

		c.addPreference(vibrationLengthTogglePref);

		EditTextPreference vibrationLengthPref = new EditTextPreference(this);
		vibrationLengthPref.setTitle("Vibration length");
		vibrationLengthPref.setSummary(s.getSharedPreferences().getString("vibrationLength", "35") + " ms");
		vibrationLengthPref.setDialogTitle("Vibration length");
		vibrationLengthPref.setDialogMessage("Enter a positive integer (ms)");
		vibrationLengthPref.setDefaultValue("35");
		vibrationLengthPref.setPersistent(true);
		vibrationLengthPref.setKey("vibrationLength");
		vibrationLengthPref.getEditText().setInputType(InputType.TYPE_CLASS_NUMBER);
		vibrationLengthPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);

		c.addPreference(vibrationLengthPref);
		
		soundLevelPref.setDependency("soundLevelToggle");		
		vibrationLengthPref.setDependency("vibrationLengthToggle");
	}
	
	@Override
    public void onConfigurationChanged(Configuration newConfig) {    	
      super.onConfigurationChanged(newConfig);
    }
	
	@Override
	protected void onResume() {
		super.onResume();
		getPreferenceScreen().getSharedPreferences().registerOnSharedPreferenceChangeListener(this);
	}

	@Override
	protected void onPause() {
		super.onPause();
		getPreferenceScreen().getSharedPreferences().unregisterOnSharedPreferenceChangeListener(this);
	}	
	
	@Override
	public void onSharedPreferenceChanged(SharedPreferences prefs, String key) {
		if(key.equals("soundLevel")) {
			getPreferenceScreen().findPreference(key).setSummary(prefs.getString(key, "error") + "%");
		}
		else if(key.equals("vibrationLength")) {
			getPreferenceScreen().findPreference(key).setSummary(prefs.getString(key, "error") + " ms");
		}
		
		else if(key.startsWith("keySound")){
			getPreferenceScreen().findPreference(key).setSummary(getPathSummary(prefs.getString(key, "")));
		}
	}
	
	@Override
	public boolean onPreferenceChange(Preference preference, Object newValue) {
		
		if(!preference.getKey().equals("soundLevel") && !preference.getKey().equals("vibrationLength"))
			return true;
		int val = -1;
		try {
			val = Integer.valueOf(newValue.toString());			
		}
		catch (Exception e) {
			Static.log(e.toString());
		}
		
		if(preference.getKey().equals("soundLevel")) {
			if(val >= 0 && val <= 100)
				return true;
		}
		else if(preference.getKey().equals("vibrationLength")) {
			if(val >= 0)
				return true;
		}
		
		return false;
	}
	
	private String getPathSummary(String val) {
		val = val.trim();
		if(val.length() == 0)
			return "Default HTC";
		else
			return val;
	}
}
