package jonasl.ime;

import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.content.res.Configuration;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.preference.PreferenceScreen;
import android.preference.Preference.OnPreferenceChangeListener;
import android.text.InputType;
import android.view.inputmethod.EditorInfo;
import android.widget.Toast;

public class PrefsTextInput extends PreferenceActivity implements OnSharedPreferenceChangeListener, OnPreferenceChangeListener {

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		PreferenceScreen s = getPreferenceManager().createPreferenceScreen(this);		
		setPreferenceScreen(s);
		
		PreferenceCategory c = new PreferenceCategory(this);
		c.setTitle("Text input");
		s.addPreference(c);
		
		CheckBoxPreference insertSpacePref = new CheckBoxPreference(this);
		insertSpacePref.setTitle("Insert spaces");
		insertSpacePref.setSummary("Inserts space after selection from dictionary");
		insertSpacePref.setDefaultValue(true);
		insertSpacePref.setPersistent(true);
		insertSpacePref.setKey("insertSpace");

		c.addPreference(insertSpacePref);
		
		CheckBoxPreference insertSpaceAfterWordPref = new CheckBoxPreference(this);
		insertSpaceAfterWordPref.setTitle("... after words");
		insertSpaceAfterWordPref.setSummary("Inserts space selecting a word");
		insertSpaceAfterWordPref.setDefaultValue(true);
		insertSpaceAfterWordPref.setPersistent(true);
		insertSpaceAfterWordPref.setKey("insertSpaceAfterWord");

		c.addPreference(insertSpaceAfterWordPref);	
		

		CheckBoxPreference insertSpaceAfterSeparatorPref = new CheckBoxPreference(this);
		insertSpaceAfterSeparatorPref.setTitle("... after separators");
		insertSpaceAfterSeparatorPref.setSummary("Inserts space after ending selection with a separator");
		insertSpaceAfterSeparatorPref.setDefaultValue(true);
		insertSpaceAfterSeparatorPref.setPersistent(true);
		insertSpaceAfterSeparatorPref.setKey("insertSpaceAfterSeparator");

		c.addPreference(insertSpaceAfterSeparatorPref);

		CheckBoxPreference insertSpaceAfterSmileyPref = new CheckBoxPreference(this);
		insertSpaceAfterSmileyPref.setTitle("... after smileys");
		insertSpaceAfterSmileyPref.setSummary("Inserts space after smileys");
		insertSpaceAfterSmileyPref.setDefaultValue(true);
		insertSpaceAfterSmileyPref.setPersistent(true);
		insertSpaceAfterSmileyPref.setKey("insertSpaceAfterSmiley");

		c.addPreference(insertSpaceAfterSmileyPref);
		
		CheckBoxPreference insertPeriodPref = new CheckBoxPreference(this);
		insertPeriodPref.setTitle("Insert periods");
		insertPeriodPref.setSummary("Insert periods by double tapping space");
		insertPeriodPref.setDefaultValue(true);
		insertPeriodPref.setPersistent(true);
		insertPeriodPref.setKey("insertPeriod");

		c.addPreference(insertPeriodPref);

		CheckBoxPreference noAutoCapPref = new CheckBoxPreference(this);
		noAutoCapPref.setTitle("Disable capitalization");
		noAutoCapPref.setDefaultValue(false);
		noAutoCapPref.setPersistent(true);
		noAutoCapPref.setKey("noAutoCap");

		c.addPreference(noAutoCapPref);

		CheckBoxPreference customDelayTogglePref = new CheckBoxPreference(this);
		customDelayTogglePref.setTitle("Custom popup delay");
		customDelayTogglePref.setDefaultValue(false);
		customDelayTogglePref.setPersistent(true);
		customDelayTogglePref.setKey("customDelayToggle");

		c.addPreference(customDelayTogglePref);

		EditTextPreference customDelayPref = new EditTextPreference(this);
		customDelayPref.setTitle("Popup delay value");
		customDelayPref.setSummary(s.getSharedPreferences().getString("customDelay", "750") + " ms");
		customDelayPref.setDialogTitle("Custom popup delay");
		customDelayPref.setDialogMessage("Integer > 100 (ms)");
		customDelayPref.setDefaultValue("750");
		customDelayPref.setPersistent(true);
		customDelayPref.setKey("customDelay");
		customDelayPref.setOnPreferenceChangeListener(this);
		customDelayPref.getEditText().setInputType(InputType.TYPE_CLASS_NUMBER);
		customDelayPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);

		c.addPreference(customDelayPref);

		CheckBoxPreference androidSmileysPref = new CheckBoxPreference(this);
		androidSmileysPref.setTitle("Android smileys");
		androidSmileysPref.setSummary("Standard Android smileys (with '-'). Kill KB to apply.");
		androidSmileysPref.setDefaultValue(true);
		androidSmileysPref.setPersistent(true);
		androidSmileysPref.setKey("androidSmileys");

		c.addPreference(androidSmileysPref);
		
		CheckBoxPreference scandHackPref = new CheckBoxPreference(this);
		scandHackPref.setTitle("Localize popup chars");
		scandHackPref.setSummary("Language specific popup chars closer to pressed key (help me w. more langs!)");
		scandHackPref.setDefaultValue(true);
		scandHackPref.setPersistent(true);
		scandHackPref.setKey("localizePopups");
		
		c.addPreference(scandHackPref);
		

		CheckBoxPreference replaceCharsPref = new CheckBoxPreference(this);
		replaceCharsPref.setTitle("Replace chars");
		replaceCharsPref.setSummary("Char in \"Forbidden\" replaced by corresponding in \"Replacement\"");
		replaceCharsPref.setDefaultValue(false);
		replaceCharsPref.setPersistent(true);
		replaceCharsPref.setKey("replaceChars");

		c.addPreference(replaceCharsPref);
		
		CheckBoxPreference replaceCharsSMSPref = new CheckBoxPreference(this);
		replaceCharsSMSPref.setTitle("... only in SMS");
		replaceCharsSMSPref.setDefaultValue(false);
		replaceCharsSMSPref.setPersistent(true);
		replaceCharsSMSPref.setKey("replaceCharsSMS");

		c.addPreference(replaceCharsSMSPref);
		
		EditTextPreference forbiddenPref = new EditTextPreference(this);
		forbiddenPref.setTitle("Forbidden chars");
		forbiddenPref.setSummary(s.getSharedPreferences().getString("forbiddenChars", "åäö"));
		forbiddenPref.setDialogTitle("Forbidden chars");
		forbiddenPref.setDialogMessage("Enter the unwanted chars without spaces. Only lower case. (Ex: 'åäö')");
		forbiddenPref.setDefaultValue("���");
		forbiddenPref.setPersistent(true);
		forbiddenPref.setKey("forbiddenChars");
		forbiddenPref.setOnPreferenceChangeListener(this);		
		forbiddenPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);
		
		c.addPreference(forbiddenPref);
		
		
		EditTextPreference replacementPrefs = new EditTextPreference(this);
		replacementPrefs.setTitle("Replacement chars");
		replacementPrefs.setSummary(s.getSharedPreferences().getString("replacementChars", "aao"));
		replacementPrefs.setDialogTitle("Replacement chars");
		replacementPrefs.setDialogMessage("Enter replacement chars without spaces. Only lower case. (Ex: 'aao')");
		replacementPrefs.setDefaultValue("aao");
		replacementPrefs.setPersistent(true);
		replacementPrefs.setKey("replacementChars");
		replacementPrefs.setOnPreferenceChangeListener(this);		
		replacementPrefs.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);
		
		c.addPreference(replacementPrefs);
		
		
		customDelayPref.setDependency("customDelayToggle");		
		insertSpaceAfterSeparatorPref.setDependency("insertSpace");
		insertSpaceAfterSmileyPref.setDependency("insertSpace");
		forbiddenPref.setDependency("replaceChars");
		replacementPrefs.setDependency("replaceChars");
		replaceCharsSMSPref.setDependency("replaceChars");
	}
	
	@Override
    public void onConfigurationChanged(Configuration newConfig) {    	
      super.onConfigurationChanged(newConfig);
    }
	
	@Override
	protected void onResume() {
		super.onResume();
		getPreferenceScreen().getSharedPreferences().registerOnSharedPreferenceChangeListener(this);
	}

	@Override
	protected void onPause() {
		super.onPause();
		getPreferenceScreen().getSharedPreferences().unregisterOnSharedPreferenceChangeListener(this);
	}	
	
	@Override
	public void onSharedPreferenceChanged(SharedPreferences prefs, String key) {
		if(key.equals("customDelay")) {		
			getPreferenceScreen().findPreference(key).setSummary(prefs.getString(key, "error") + " ms");
		}
		else if(key.equals("forbiddenChars") || key.equals("replacementChars")) {
			String forbidden = getPreferenceScreen().getSharedPreferences().getString("forbiddenChars", "");
			String replacement = getPreferenceScreen().getSharedPreferences().getString("replacementChars", "");
			
			getPreferenceScreen().findPreference(key).setSummary(prefs.getString(key, "error"));
			
			String error = "";
			
			if(forbidden.length() == 0)
				error += "Forbidden is empty\n";
			
			if(replacement.length() == 0)
				error += "Replacement is empty\n";
			
			if(forbidden.contains(" "))
				error += "Forbidden has space(s)\n";
			
			if(replacement.contains(" "))
				error += "Replacement has space(s)\n";
			
			if(replacement.length() != forbidden.length())
				error += "Forbidden and Replacement not equal length";
			
			if(error.length() > 0) {
				error = "Replace char config error:\n" + error;
				Toast.makeText(this, error.trim(), Toast.LENGTH_LONG).show();
			}
			else {
				Toast.makeText(this, "Current replace char config is valid", Toast.LENGTH_LONG).show();
			}
		}
	}

	@Override
	public boolean onPreferenceChange(Preference preference, Object newValue) {
		
		if(!preference.getKey().equals("customDelay"))
			return true;
		
		try {
			int val = Integer.valueOf(newValue.toString());
			if(val >= 100)
				return true;
		}
		catch (Exception e) {
			Static.log(e.toString());
		}
		return false;
	}
}
