package jonasl.ime;

import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import android.content.res.Configuration;
import android.os.Bundle;
import android.preference.CheckBoxPreference;
import android.preference.EditTextPreference;
import android.preference.ListPreference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.preference.PreferenceScreen;
import android.text.InputType;
import android.view.inputmethod.EditorInfo;

public class PrefsUI extends PreferenceActivity implements OnSharedPreferenceChangeListener  {
	
	public static String[] entries = {"No action", "Move cursor", "Dpad up/down", "Change language"};
	public static String[] values = {"none", "move", "arrow", "lang"};
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		SharedPreferences prefs = getPreferenceManager().getSharedPreferences();
		PreferenceScreen s = getPreferenceManager().createPreferenceScreen(this);		
		setPreferenceScreen(s);
		
		PreferenceCategory c = new PreferenceCategory(this);
		c.setTitle("User interface");
		s.addPreference(c);
		
		CheckBoxPreference barPref = new CheckBoxPreference(this);
		barPref.setTitle("Show candidates bar");
		barPref.setSummary("Custom imitation of stock keyboard");
		barPref.setDefaultValue(true);
		barPref.setPersistent(true);
		barPref.setKey("showBar");

		c.addPreference(barPref);

		CheckBoxPreference leftAlignPref = new CheckBoxPreference(this);
		leftAlignPref.setTitle("Left align candidates");
		leftAlignPref.setSummary("Standard is center alignment");
		leftAlignPref.setDefaultValue(false);
		leftAlignPref.setPersistent(true);
		leftAlignPref.setKey("leftAlign");

		c.addPreference(leftAlignPref);
		
		CheckBoxPreference showLangPref = new CheckBoxPreference(this);
		showLangPref.setTitle("Show language at start");
		showLangPref.setSummary("Shows current language when input starts");
		showLangPref.setDefaultValue(false);
		showLangPref.setPersistent(true);
		showLangPref.setKey("showLang");

		c.addPreference(showLangPref);
		
		CheckBoxPreference altSymbPref = new CheckBoxPreference(this);
		altSymbPref.setTitle("Alt. symbol keyboard");
		altSymbPref.setSummary("Use phone keypad/compact QWERTY symbol kb. also for QWERTY");
		altSymbPref.setDefaultValue(false);
		altSymbPref.setPersistent(true);
		altSymbPref.setKey("altSymb");

		c.addPreference(altSymbPref);
		
		
		CheckBoxPreference spaceHackPref = new CheckBoxPreference(this);
		spaceHackPref.setTitle("Space hack");
		spaceHackPref.setSummary("Enable space bar in URL KB. Kill KB to apply changes.");
		spaceHackPref.setDefaultValue(true);
		spaceHackPref.setPersistent(true);
		spaceHackPref.setKey("spaceHack");

		c.addPreference(spaceHackPref);

		CheckBoxPreference smileyHackPref = new CheckBoxPreference(this);
		smileyHackPref.setTitle("Smiley hack");
		smileyHackPref.setSummary("Smileys instead of hide button. Kill KB to apply changes.");
		smileyHackPref.setDefaultValue(true);
		smileyHackPref.setPersistent(true);
		smileyHackPref.setKey("smileyHack");

		c.addPreference(smileyHackPref);
		
		CheckBoxPreference swapSmileysPref = new CheckBoxPreference(this);
		swapSmileysPref.setTitle("Smiley/12# swap");
		swapSmileysPref.setSummary("Not great with all custom smileys. Kill KB to apply.");
		swapSmileysPref.setDefaultValue(false);
		swapSmileysPref.setPersistent(true);
		swapSmileysPref.setKey("swapSmileys");

		c.addPreference(swapSmileysPref);		
		
		CheckBoxPreference customSmileysTogglePref = new CheckBoxPreference(this);
		customSmileysTogglePref.setTitle("Custom smileys");
		customSmileysTogglePref.setSummary("Kill KB to apply any changes to smileys.");
		customSmileysTogglePref.setDefaultValue(false);
		customSmileysTogglePref.setPersistent(true);
		customSmileysTogglePref.setKey("customSmileysToggle");

		c.addPreference(customSmileysTogglePref);
		
		
		EditTextPreference customSmileysPref = new EditTextPreference(this);
		customSmileysPref.setTitle("Edit smileys");
		customSmileysPref.setSummary(s.getSharedPreferences().getString("customSmileys", ":-) :-("));
		customSmileysPref.setDialogTitle("Custom smileys");
		customSmileysPref.setDialogMessage("At least two smileys separated by space");
		customSmileysPref.setDefaultValue(":-) :-(");
		customSmileysPref.setPersistent(true);
		customSmileysPref.setKey("customSmileys");
		customSmileysPref.getEditText().setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PHONETIC);
		customSmileysPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);
		
		c.addPreference(customSmileysPref);
		
		CheckBoxPreference customDomainsTogglePref = new CheckBoxPreference(this);
		customDomainsTogglePref.setTitle("Custom domains");
		customDomainsTogglePref.setSummary("Kill KB to apply any changes to domains.");
		customDomainsTogglePref.setDefaultValue(false);
		customDomainsTogglePref.setPersistent(true);
		customDomainsTogglePref.setKey("customDomainsToggle");

		c.addPreference(customDomainsTogglePref);
		
		
		EditTextPreference popupDomainsPref = new EditTextPreference(this);
		popupDomainsPref.setTitle("Popup domains");
		popupDomainsPref.setSummary(s.getSharedPreferences().getString("popupDomains", ".net .com .org .edu"));
		popupDomainsPref.setDialogTitle("Popup domains");
		popupDomainsPref.setDialogMessage("Enter domains separated by space. About four is best.");
		popupDomainsPref.setDefaultValue(".net .com .org .edu");
		popupDomainsPref.setPersistent(true);
		popupDomainsPref.setKey("popupDomains");
		popupDomainsPref.getEditText().setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PHONETIC);
		popupDomainsPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);
		
		c.addPreference(popupDomainsPref);
		
		EditTextPreference keyDomainPref = new EditTextPreference(this);
		keyDomainPref.setTitle("Key domain");
		keyDomainPref.setSummary(s.getSharedPreferences().getString("keyDomain", ".com"));
		keyDomainPref.setDialogTitle("Key domain");
		keyDomainPref.setDialogMessage("Key domain");
		keyDomainPref.setDefaultValue(".com");
		keyDomainPref.setPersistent(true);
		keyDomainPref.setKey("keyDomain");
		keyDomainPref.getEditText().setInputType(InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_VARIATION_PHONETIC);
		keyDomainPref.getEditText().setImeOptions(EditorInfo.IME_ACTION_NONE);
		
		c.addPreference(keyDomainPref);

		CheckBoxPreference disablePreviewPref = new CheckBoxPreference(this);
		disablePreviewPref.setTitle("No key previews");
		disablePreviewPref.setSummary("Disables the key preview popup");
		disablePreviewPref.setDefaultValue(false);
		disablePreviewPref.setPersistent(true);
		disablePreviewPref.setKey("disablePreview");

		c.addPreference(disablePreviewPref);
		
		ListPreference qwertyWidthAdjustPref = new ListPreference(this);
		qwertyWidthAdjustPref.setTitle("Port. QWERTY key adjust");
		qwertyWidthAdjustPref.setSummary(prefs.getString("qwertyWidthAdjust", "0") + "px. Kill KB to apply changes.");
		qwertyWidthAdjustPref.setDefaultValue("0");
		qwertyWidthAdjustPref.setPersistent(true);
		qwertyWidthAdjustPref.setKey("qwertyWidthAdjust");
		qwertyWidthAdjustPref.setEntries(new String[] {"0px", "1px", "2px", "3px", "4px", "5px", "6px", "7px", "8px", "9px", "10px"});
		qwertyWidthAdjustPref.setEntryValues(new String[] {"0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10"});
		
		c.addPreference(qwertyWidthAdjustPref);
		
		ListPreference volumePref = new ListPreference(this);
		volumePref.setTitle("Volume keys");
		volumePref.setSummary(getEntry(prefs.getString("volumeKeys", "none")));
		volumePref.setDefaultValue("none");
		volumePref.setPersistent(true);
		volumePref.setKey("volumeKeys");
		volumePref.setEntries(entries);
		volumePref.setEntryValues(values);		

		c.addPreference(volumePref);	
		
		CheckBoxPreference enterSendPref = new CheckBoxPreference(this);
		enterSendPref.setTitle("Enter to send");
		enterSendPref.setSummary("Only for stock messaging app");
		enterSendPref.setDefaultValue(false);
		enterSendPref.setPersistent(true);
		enterSendPref.setKey("enterSend");

		c.addPreference(enterSendPref);
		
		customSmileysTogglePref.setDependency("smileyHack");
		swapSmileysPref.setDependency("smileyHack");
		customSmileysPref.setDependency("customSmileysToggle");
		
		popupDomainsPref.setDependency("customDomainsToggle");
		keyDomainPref.setDependency("customDomainsToggle");
	}
	
	
	@Override
    public void onConfigurationChanged(Configuration newConfig) {    	
      super.onConfigurationChanged(newConfig);
    }
	
	@Override
	protected void onResume() {
		super.onResume();
		getPreferenceScreen().getSharedPreferences().registerOnSharedPreferenceChangeListener(this);
	}

	@Override
	protected void onPause() {
		super.onPause();
		getPreferenceScreen().getSharedPreferences().unregisterOnSharedPreferenceChangeListener(this);
	}
	
	public void onSharedPreferenceChanged(SharedPreferences prefs, String key) {
		if(key.equals("customSmileys") || key.equals("keyDomain") || key.equals("popupDomains"))
			getPreferenceScreen().findPreference(key).setSummary(prefs.getString(key, ""));
		else if(key.equals("volumeKeys"))
			getPreferenceScreen().findPreference(key).setSummary(getEntry(prefs.getString(key, "")));		
		else if(key.equals("qwertyWidthAdjust"))
			getPreferenceScreen().findPreference(key).setSummary(prefs.getString("qwertyWidthAdjust", "0") + "px. Kill KB to apply changes.");
		
	}
	
	private String getEntry(String value) {
		for(int i = 0; i < values.length; i++) {
			if(values[i].equals(value))
				return entries[i];
		}
		
		return "Internal error";
	}
	
}