package jonasl.ime;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Scanner;

import android.app.ProgressDialog;
import android.content.Context;
import android.content.res.Configuration;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.preference.CheckBoxPreference;
import android.preference.Preference;
import android.preference.PreferenceActivity;
import android.preference.PreferenceCategory;
import android.preference.PreferenceScreen;
import android.widget.Toast;

public class PrefsUserDict extends PreferenceActivity {

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		
		PreferenceScreen s = getPreferenceManager().createPreferenceScreen(this);		
		setPreferenceScreen(s);
		
		PreferenceCategory c = new PreferenceCategory(this);
		c.setTitle("User dictionary");
		s.addPreference(c);
		
		Preference exportPref = new Preference(this);
		exportPref.setTitle("Export dictionary");
		exportPref.setSummary("Export user dictionary to /sdcard/htcimedict.txt");
		exportPref.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
					public boolean onPreferenceClick(Preference p) {
						Message msg = Message.obtain(exportHandler, 0, (PreferenceActivity)p.getContext());
						exportHandler.sendMessage(msg);
						return true;
					}
				});

		c.addPreference(exportPref);

		Preference importPref = new Preference(this);
		importPref.setTitle("Import dictionary");
		importPref.setSummary("Import user dictionary from /sdcard/htcimedict.txt avoiding duplicates");
		importPref.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
					public boolean onPreferenceClick(Preference p) {
						Message msg = Message.obtain(importHandler, 0, (PreferenceActivity)p.getContext());
						importHandler.sendMessage(msg);
						return true;
					}
				});

		c.addPreference(importPref);
		
		CheckBoxPreference clearTogglePref = new CheckBoxPreference(this);
		clearTogglePref.setTitle("Enable clear dictionary");
		clearTogglePref.setDefaultValue(false);
		clearTogglePref.setPersistent(false);
		clearTogglePref.setKey("clearToggle");

		c.addPreference(clearTogglePref);
		
		Preference clearPref = new Preference(this);
		clearPref.setTitle("Clear dictionary");
		clearPref.setSummary("Clear user dictionary without confirmation");
		clearPref.setOnPreferenceClickListener(new Preference.OnPreferenceClickListener() {
					public boolean onPreferenceClick(Preference p) {
						Message msg = Message.obtain(clearHandler, 0, (PreferenceActivity)p.getContext());
						clearHandler.sendMessage(msg);
						return true;
					}
				});

		c.addPreference(clearPref);
		
		clearPref.setDependency("clearToggle");
	}
	
	@Override
    public void onConfigurationChanged(Configuration newConfig) {    	
      super.onConfigurationChanged(newConfig);
    }
	
	private Handler exportHandler = new Handler() {

		public void handleMessage(Message msg) {
			switch (msg.what) {
			case 0:
				if (exportContext != null)
					break;

				exportContext = (Context) msg.obj;
				exportProgress = ProgressDialog.show(exportContext,
						"Export user dictionary", "Don't abort!", true, false);
				
				startExportThread();

				break;

			case 1:
				exportProgress.setMessage("Reading " + msg.arg1
						+ " existing words");
				break;			
			case 2:
				float p = (float)msg.arg1 / (float)msg.arg2;
				p = p * 100;
				
				exportProgress.setMessage("Processing word " + msg.arg1
						+ " of " + msg.arg2 + " (" + (int)Math.floor(p) + "%)");
				break;
				
			case 4:				
				exportProgress.dismiss();
				
				Toast t1 = Toast.makeText(exportContext, msg.arg1 + " words exported",
						Toast.LENGTH_SHORT);
				t1.show();				
				exportContext = null;
				break;
				
			case 5:
				exportProgress.dismiss();
				
				Toast t2 = Toast.makeText(exportContext, "Export failed: "
						+ msg.obj.toString(), Toast.LENGTH_LONG);
				t2.show();
				exportContext = null;
				break;
			case 6:
				if(exportProgress != null) {
					exportProgress.dismiss();
					exportProgress = null;
				}
				
				exportContext = null;
				break;
			}
		}
	};

	public void startExportThread() {
		new Thread() {
			public void run() {
				exportUserDictionary();
			}
		}.start();
	}

	Context exportContext = null;
	ProgressDialog exportProgress = null;

	public void exportUserDictionary() {

		FileOutputStream fos = null;
		OutputStreamWriter osw = null;

		try {
			Thread.sleep(500);
			Class<?> c = Class
					.forName("com.htc.android.htcime.XT9IME.util.XT9UDBHandler");
			Constructor<?> ctor = c.getDeclaredConstructors()[0];
			Object udbHandler = ctor.newInstance(exportContext);
			Method wordCountMethod = null;
			Method getAllWordsMethod = null;

			for (Method m : c.getMethods()) {
				if (m.getName().equals("getWordAll"))
					getAllWordsMethod = m;

				if (m.getName().equals("getWordCount"))
					wordCountMethod = m;
			}

			int wordCount = (Integer) wordCountMethod.invoke(udbHandler, (Object[]) null);
			
			Message msg = Message.obtain(exportHandler, 1, wordCount, 0);
			exportHandler.sendMessage(msg);
			
			char[] buf = new char[(wordCount + 1) * 64 - 1];
			int len = (Integer) getAllWordsMethod.invoke(udbHandler, buf);

			String allWordsStr = new String(buf, 0, len);
			String[] allWords = allWordsStr.split("\\|");
			Arrays.sort(allWords);
			
			Thread.sleep(300);

			fos = new FileOutputStream(new File(android.os.Environment
					.getExternalStorageDirectory(), "htcimedict.txt"));
			osw = new OutputStreamWriter(fos, "UTF-8");
			int cnt = 0;
			for (String word : allWords) {
				cnt++;
				msg = Message.obtain(exportHandler, 2, cnt, allWords.length);
				exportHandler.sendMessage(msg);				
				osw.append(word);
				osw.append("\n");
			}

			osw.flush();
			Thread.sleep(300);
			msg = Message.obtain(exportHandler, 4, cnt, wordCount);
			exportHandler.sendMessage(msg);
		} catch (Exception e) {
			Static.log("exportUserDictionary: " + e.toString());

			Message msg = Message.obtain(exportHandler, 5, e);
			exportHandler.sendMessage(msg);
		} finally {
			if (osw != null) {
				try {
					osw.close();
				} catch (Exception e) {
				}
			}

			if (fos != null) {
				try {
					fos.close();
				} catch (Exception e) {
				}
			}
			
			Message msg = Message.obtain(exportHandler, 6);
			exportHandler.sendMessage(msg);
		}
	}

	private Handler importHandler = new Handler() {

		public void handleMessage(Message msg) {
			switch (msg.what) {
			case 0:
				if (importContext != null)
					break;

				importContext = (Context) msg.obj;
				importProgress = ProgressDialog.show(importContext,
						"Import user dictionary", "Don't abort!", true, false);
				
				startImportThread();

				break;

			case 1:
				importProgress.setMessage("Reading " + msg.arg1
						+ " existing words");
				break;
			case 2:
				importProgress.setMessage("Reading words from SD (" + msg.arg1 + ")");
				break;
			case 3:
				float p = (float)msg.arg1 / (float)msg.arg2;
				p = p * 100;
				
				importProgress.setMessage("Processing word " + msg.arg1
						+ " of " + msg.arg2 + " (" + (int)Math.floor(p) + "%)");
				break;
				
			case 4:
				importProgress.dismiss();
				
				Toast t1 = Toast.makeText(importContext, msg.arg1 + " words imported",
						Toast.LENGTH_SHORT);
				t1.show();				
				importContext = null;
				break;
				
			case 5:
				importProgress.dismiss();
				
				Toast t2 = Toast.makeText(importContext, "Import failed: "
						+ msg.obj.toString(), Toast.LENGTH_LONG);
				t2.show();
				importContext = null;
				break;
			case 6:
				if(importProgress != null) {
					importProgress.dismiss();
					importProgress = null;
				}
				
				importContext = null;
				break;
			}
		}
	};

	public void startImportThread() {
		new Thread() {
			public void run() {
				importUserDictionary();
			}
		}.start();
	}

	Context importContext = null;
	ProgressDialog importProgress = null;

	public void importUserDictionary() {

		Scanner scanner = null;

		try {
			
			Thread.sleep(500);

			Class<?> c = Class
					.forName("com.htc.android.htcime.XT9IME.util.XT9UDBHandler");
			Constructor<?> ctor = c.getDeclaredConstructors()[0];
			Object udbHandler = ctor.newInstance(importContext);
			Method wordCountMethod = null;
			Method getAllWordsMethod = null;
			Method addWordMethod = null;

			for (Method m : c.getMethods()) {
				if (m.getName().equals("getWordAll"))
					getAllWordsMethod = m;

				if (m.getName().equals("getWordCount"))
					wordCountMethod = m;

				if (m.getName().equals("addWord"))
					addWordMethod = m;
			}

			int wordCount = (Integer) wordCountMethod.invoke(udbHandler, (Object[]) null);
			
			Message msg = Message.obtain(importHandler, 1, wordCount, 0);
			importHandler.sendMessage(msg);

			char[] buf = new char[(wordCount + 1) * 64 - 1];
			int len = (Integer) getAllWordsMethod.invoke(udbHandler, buf);

			String allWordsStr = new String(buf, 0, len);
			String[] allWords = allWordsStr.split("\\|");

			HashMap<String, String> existing = new HashMap<String, String>();
			for (String word : allWords)
				existing.put(word, null);
			
			Thread.sleep(300);			
			
			scanner = new Scanner(new File(android.os.Environment
					.getExternalStorageDirectory(), "htcimedict.txt"), "UTF-8");
			ArrayList<String> newWords = new ArrayList<String>();
			while (scanner.hasNextLine()) {
				String newWord = scanner.nextLine();

				if (newWord == null || newWord.length() < 1) {					
					continue;
				}

				if (newWord.length() > 64)
					continue;
				newWords.add(newWord);
				
				msg = Message.obtain(importHandler, 2, newWords.size(), 0);
				importHandler.sendMessage(msg);
			}
			
			Thread.sleep(300);
			
			int cnt = 0;
			int imported = 0;

			for(String newWord : newWords) {
				cnt++;
				msg = Message.obtain(importHandler, 3, cnt, newWords.size());
				importHandler.sendMessage(msg);
				
				if (existing.containsKey(newWord)) {					
					continue;
				}
				char[] tmp = newWord.toCharArray();
				if ((Integer) addWordMethod.invoke(udbHandler, new Object[] {
						tmp, tmp.length }) == 0) {
					imported++;
					existing.put(newWord, null);
				} else {
				}
			}
			Static.log(imported + " words imported");
			
			Thread.sleep(300);
			msg = Message.obtain(importHandler, 4, imported, 0);
			importHandler.sendMessage(msg);
		} catch (Exception e) {
			Static.log("importUserDictionary fail: " + e.toString());
			e.printStackTrace();
			Message msg = Message.obtain(importHandler, 5, e);
			importHandler.sendMessage(msg);
		} finally {
			if (scanner != null) {
				try {
					scanner.close();
				} catch (Exception e) {
				}
			}
			
			Message msg = Message.obtain(importHandler, 6);
			importHandler.sendMessage(msg);
		}
	}
	
	
	private Handler clearHandler = new Handler() {

		public void handleMessage(Message msg) {
			switch (msg.what) {
			case 0:
				if (clearContext != null)
					break;

				clearContext = (Context) msg.obj;
				clearProgress = ProgressDialog.show(clearContext,
						"Clearing user dictionary", "Don't abort!", true, false);
				
				startClearThread();

				break;

			case 1:
				clearProgress.setMessage("Reading " + msg.arg1
						+ " existing words");
				break;			
			case 2:
				float p = (float)msg.arg1 / (float)msg.arg2;
				p = p * 100;
				
				clearProgress.setMessage("Processing word " + msg.arg1
						+ " of " + msg.arg2 + " (" + (int)Math.floor(p) + "%)");
				break;
				
			case 4:				
				clearProgress.dismiss();
				
				Toast t1 = Toast.makeText(clearContext, msg.arg1 + " words removed",
						Toast.LENGTH_SHORT);
				t1.show();				
				exportContext = null;
				break;
				
			case 5:
				clearProgress.dismiss();
				
				Toast t2 = Toast.makeText(clearContext, "Clear failed: "
						+ msg.obj.toString(), Toast.LENGTH_LONG);
				t2.show();
				exportContext = null;
				break;
			case 6:
				if(clearProgress != null) {
					clearProgress.dismiss();
					clearProgress = null;
				}
				
				clearContext = null;
				break;
			}
		}
	};

	public void startClearThread() {
		new Thread() {
			public void run() {
				clearUserDictionary();
			}
		}.start();
	}

	Context clearContext = null;
	ProgressDialog clearProgress = null;

	public void clearUserDictionary() {

		try {
			
			Thread.sleep(500);

			Class<?> c = Class
					.forName("com.htc.android.htcime.XT9IME.util.XT9UDBHandler");
			Constructor<?> ctor = c.getDeclaredConstructors()[0];
			Object udbHandler = ctor.newInstance(clearContext);
			Method wordCountMethod = null;
			Method getAllWordsMethod = null;
			Method delWordMethod = null;

			for (Method m : c.getMethods()) {
				if (m.getName().equals("getWordAll"))
					getAllWordsMethod = m;

				if (m.getName().equals("getWordCount"))
					wordCountMethod = m;

				if (m.getName().equals("delWord"))
					delWordMethod = m;
			}

			int wordCount = (Integer) wordCountMethod.invoke(udbHandler, (Object[]) null);
			
			Message msg = Message.obtain(clearHandler, 1, wordCount, 0);
			clearHandler.sendMessage(msg);
			
			char[] buf = new char[(wordCount + 1) * 64 - 1];
			int len = (Integer) getAllWordsMethod.invoke(udbHandler, buf);

			String allWordsStr = new String(buf, 0, len);
			String[] allWords = allWordsStr.split("\\|");
			
			Thread.sleep(300);
			
			int cnt = 0;
			if(wordCount > 0) {
				for(String w : allWords) {
					cnt++;
					delWordMethod.invoke(udbHandler, w.toCharArray());
					
					msg = Message.obtain(clearHandler, 2, cnt, allWords.length);
					clearHandler.sendMessage(msg);
				}
			}
			
			Thread.sleep(300);
			
			msg = Message.obtain(clearHandler, 4, cnt, 0);
			clearHandler.sendMessage(msg);
			
		}
		catch(Exception e) {
			Static.log("clearUserDictionary fail: " + e.toString());

			Message msg = Message.obtain(clearHandler, 5, 4);
			clearHandler.sendMessage(msg);
		}
		finally {
			Message msg = Message.obtain(clearHandler, 6);
			clearHandler.sendMessage(msg);
		}
	}
}
